@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: caudium.c,v 1.10 2001/02/26 06:07:36 andi Exp $ */
+/* $Id: caudium.c,v 1.22.2.1 2001/10/12 11:56:37 ssb Exp $ */
 
 #include "php.h"
 #ifdef HAVE_CAUDIUM
@@ -37,10 +37,13 @@
  *
  */
 #define NO_PIKE_SHORTHAND
-/* Define to run w/o th_farm. This allows for A LOT easier
- * debugging due to stupid gdb...
+
+/* Ok, we are now using Pike level threads to handle PHP4 since
+ * the nice th_farm threads aren't working on Linux with glibc 2.2
+ * (why this is I don't know).
  */
-/*#define TEST_NO_THREADS*/
+#define USE_PIKE_LEVEL_THREADS
+
 #include <fdlib.h>
 #include <program.h>
 #include <pike_types.h>
@@ -89,13 +92,13 @@ typedef struct
   struct pike_string *filename;
   int my_fd;
   int written;
-  SLS_D;
+  TSRMLS_D;
 } php_caudium_request;
 
 
 void pike_module_init(void);
 void pike_module_exit(void);
-static void free_struct(SLS_D);
+static void free_struct(TSRMLS_D);
 void f_php_caudium_request_handler(INT32 args);
 
 /* Defines to get to the data supplied when the script is started. */
@@ -201,10 +204,9 @@ INLINE static int lookup_integer_header(char *headername, int default_value)
  */
 
 INLINE static int
-php_caudium_low_ub_write(const char *str, uint str_length) {
+php_caudium_low_ub_write(const char *str, uint str_length TSRMLS_DC) {
   int sent_bytes = 0;
   struct pike_string *to_write = NULL;
-  PLS_FETCH();
   GET_THIS();
   if(!MY_FD_OBJ->prog) {
     PG(connection_status) = PHP_CONNECTION_ABORTED;
@@ -232,9 +234,8 @@ php_caudium_low_ub_write(const char *str, uint str_length) {
  */
 
 static int
-php_caudium_sapi_ub_write(const char *str, uint str_length)
+php_caudium_sapi_ub_write(const char *str, uint str_length TSRMLS_DC)
 {
-  PLS_FETCH();
   GET_THIS();
   int sent_bytes = 0, fd = MY_FD;
   if(fd)
@@ -263,7 +264,7 @@ php_caudium_sapi_ub_write(const char *str, uint str_length)
     }
     THIS->written += sent_bytes;
   } else {
-    THREAD_SAFE_RUN(sent_bytes = php_caudium_low_ub_write(str, str_length),
+    THREAD_SAFE_RUN(sent_bytes = php_caudium_low_ub_write(str, str_length TSRMLS_CC),
 		    "write");
   }
   return sent_bytes;
@@ -329,7 +330,7 @@ php_caudium_set_header(char *header_name, char *value, char *p)
  */
 static int
 php_caudium_sapi_header_handler(sapi_header_struct *sapi_header,
-			      sapi_headers_struct *sapi_headers SLS_DC)
+			      sapi_headers_struct *sapi_headers TSRMLS_DC)
 {
   char *header_name, *header_content, *p;
   header_name = sapi_header->header;
@@ -351,9 +352,8 @@ php_caudium_sapi_header_handler(sapi_header_struct *sapi_header,
  */
 
 INLINE static int
-php_caudium_low_send_headers(sapi_headers_struct *sapi_headers SLS_DC)
+php_caudium_low_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 {
-  PLS_FETCH();
   struct pike_string *ind;
   struct svalue *s_headermap;
   GET_THIS();
@@ -378,10 +378,10 @@ php_caudium_low_send_headers(sapi_headers_struct *sapi_headers SLS_DC)
 }
 
 static int
-php_caudium_sapi_send_headers(sapi_headers_struct *sapi_headers SLS_DC)
+php_caudium_sapi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 {
   int res = 0;
-  THREAD_SAFE_RUN(res = php_caudium_low_send_headers(sapi_headers SLS_CC), "send headers");
+  THREAD_SAFE_RUN(res = php_caudium_low_send_headers(sapi_headers TSRMLS_CC), "send headers");
   return res;
 }
 
@@ -394,7 +394,7 @@ INLINE static int php_caudium_low_read_post(char *buf, uint count_bytes)
 {
   uint total_read = 0;
   GET_THIS();
-  PLS_FETCH();
+  TSRMLS_FETCH();
   
   if(!MY_FD_OBJ->prog)
   {
@@ -415,7 +415,7 @@ INLINE static int php_caudium_low_read_post(char *buf, uint count_bytes)
 }
 
 static int
-php_caudium_sapi_read_post(char *buf, uint count_bytes SLS_DC)
+php_caudium_sapi_read_post(char *buf, uint count_bytes TSRMLS_DC)
 {
   uint total_read = 0;
   THREAD_SAFE_RUN(total_read = php_caudium_low_read_post(buf, count_bytes), "read post");
@@ -428,7 +428,7 @@ php_caudium_sapi_read_post(char *buf, uint count_bytes SLS_DC)
  */
 	
 static char *
-php_caudium_sapi_read_cookies(SLS_D)
+php_caudium_sapi_read_cookies(TSRMLS_D)
 {
   char *cookies;
   cookies = lookup_string_header("HTTP_COOKIE", NULL);
@@ -439,7 +439,7 @@ static void php_info_caudium(ZEND_MODULE_INFO_FUNC_ARGS)
 {
   /*  char buf[512]; */
   php_info_print_table_start();
-  php_info_print_table_row(2, "SAPI module version", "$Id: caudium.c,v 1.10 2001/02/26 06:07:36 andi Exp $");
+  php_info_print_table_row(2, "SAPI module version", "$Id: caudium.c,v 1.22.2.1 2001/10/12 11:56:37 ssb Exp $");
   /*  php_info_print_table_row(2, "Build date", Ns_InfoBuildDate());
       php_info_print_table_row(2, "Config file path", Ns_InfoConfigFile());
       php_info_print_table_row(2, "Error Log path", Ns_InfoErrorLog());
@@ -460,6 +460,7 @@ static void php_info_caudium(ZEND_MODULE_INFO_FUNC_ARGS)
 }
 
 static zend_module_entry php_caudium_module = {
+  STANDARD_MODULE_HEADER,
   "Caudium",
   NULL,
   NULL,
@@ -467,10 +468,62 @@ static zend_module_entry php_caudium_module = {
   NULL,
   NULL,
   php_info_caudium,
+  NULL,
   STANDARD_MODULE_PROPERTIES
 };
 
 
+INLINE static void low_sapi_caudium_register_variables(zval *track_vars_array TSRMLS_DC)   
+{
+  char *tmp;
+  php_register_variable("PHP_SELF", SG(request_info).request_uri,
+			track_vars_array TSRMLS_CC);
+  php_register_variable("GATEWAY_INTERFACE", "CGI/1.1",
+			track_vars_array TSRMLS_CC);
+  php_register_variable("REQUEST_METHOD",
+			(char *) SG(request_info).request_method,
+			track_vars_array TSRMLS_CC);
+  php_register_variable("REQUEST_URI", SG(request_info).request_uri,
+			track_vars_array TSRMLS_CC);
+  php_register_variable("PATH_TRANSLATED", SG(request_info).path_translated,
+			track_vars_array TSRMLS_CC);
+
+  if( (tmp = lookup_string_header("SERVER_NAME", NULL)) != NULL)
+    php_register_variable("SERVER_NAME", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("SERVER_PORT", NULL)) != NULL)
+    php_register_variable("SERVER_PORT", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("SERVER_PROTOCOL", NULL)) != NULL)
+    php_register_variable("SERVER_PROTOCOL", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("SCRIPT_NAME", NULL)) != NULL)
+    php_register_variable("SCRIPT_NAME", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("SCRIPT_FILENAME", NULL)) != NULL)
+    php_register_variable("SCRIPT_FILENAME", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("REMOTE_ADDR", NULL)) != NULL)
+    php_register_variable("REMOTE_ADDR", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("REMOTE_PORT", NULL)) != NULL)
+    php_register_variable("REMOTE_PORT", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("DOCUMENT_ROOT", NULL)) != NULL)
+    php_register_variable("DOCUMENT_ROOT", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("HTTP_CONNECTION", NULL)) != NULL)
+    php_register_variable("HTTP_CONNECTION", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("HTTP_USER_AGENT", NULL)) != NULL)
+    php_register_variable("HTTP_USER_AGENT", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("DOCUMENT_ROOT", NULL)) != NULL)
+    php_register_variable("DOCUMENT_ROOT", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("QUERY_STRING", "")) != NULL)
+    php_register_variable("QUERY_STRING", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("REMOTE_USER", NULL)) != NULL)
+    php_register_variable("REMOTE_USER", tmp, track_vars_array TSRMLS_CC);
+  if( (tmp = lookup_string_header("REMOTE_PASSWORD", NULL)) != NULL)
+    php_register_variable("REMOTE_PASSWORD", tmp, track_vars_array TSRMLS_CC);
+  
+}
+
+static void sapi_caudium_register_variables(zval *track_vars_array TSRMLS_DC)
+{
+  THREAD_SAFE_RUN(low_sapi_caudium_register_variables(track_vars_array TSRMLS_CC), "register_variables");
+}
+
 /* this structure is static (as in "it does not change") */
 static sapi_module_struct caudium_sapi_module = {
   "caudium",
@@ -489,7 +542,7 @@ static sapi_module_struct caudium_sapi_module = {
   NULL,					/* send header handler */
   php_caudium_sapi_read_post,		/* read POST data */
   php_caudium_sapi_read_cookies,	/* read cookies */
-  NULL,					/* register server variables */
+  sapi_caudium_register_variables,	/* register server variables */
   NULL,					/* Log message */
   NULL,					/* Block interruptions */
   NULL,					/* Unblock interruptions */
@@ -502,7 +555,7 @@ static sapi_module_struct caudium_sapi_module = {
  * with a number of variables. HTTP_* variables are created for
  * the HTTP header data, so that a script can access these.
  */
-static void php_caudium_hash_environment(CLS_D ELS_DC PLS_DC SLS_DC)
+static void php_caudium_hash_environment(TSRMLS_D)
 {
   int i;
   char buf[512];
@@ -553,14 +606,12 @@ static void php_caudium_hash_environment(CLS_D ELS_DC PLS_DC SLS_DC)
 static void php_caudium_module_main(php_caudium_request *ureq)
 {
   int res;
-  int cnt = 0;
   zend_file_handle file_handle;
+#ifndef USE_PIKE_LEVEL_THREADS
   struct thread_state *state;
   extern struct program *thread_id_prog;
-  SLS_FETCH();
-  CLS_FETCH();
-  PLS_FETCH();
-  ELS_FETCH();
+#endif
+  TSRMLS_FETCH();
   GET_THIS();
   THIS->filename = ureq->filename;
   THIS->done_cb = ureq->done_cb;
@@ -569,30 +620,32 @@ static void php_caudium_module_main(php_caudium_request *ureq)
   THIS->request_data = ureq->request_data;
   free(ureq);
 
-#ifndef TEST_NO_THREADS
+#ifndef USE_PIKE_LEVEL_THREADS
   mt_lock_interpreter();
   init_interpreter();
 #if PIKE_MAJOR_VERSION == 7 && PIKE_MINOR_VERSION < 1
+  thread_id = low_clone(thread_id_prog);
+  state = OBJ2THREAD(thread_id);
   Pike_stack_top=((char *)&state)+ (thread_stack_size-16384) * STACK_DIRECTION;
   recoveries = NULL;
-  thread_id = low_clone(thread_id_prog);
   call_c_initializers(thread_id);
-  SWAP_OUT_THREAD(OBJ2THREAD(thread_id));
-  OBJ2THREAD(thread_id)->swapped=0;
   OBJ2THREAD(thread_id)->id=th_self();
   num_threads++;
   thread_table_insert(thread_id);
+  state->status=THREAD_RUNNING;
 #else
+  Pike_interpreter.thread_id = low_clone(thread_id_prog);
+  state = OBJ2THREAD(Pike_interpreter.thread_id);
   Pike_interpreter.stack_top=((char *)&state)+ (thread_stack_size-16384) * STACK_DIRECTION;
   Pike_interpreter.recoveries = NULL;
-  Pike_interpreter.thread_id = low_clone(thread_id_prog);
   call_c_initializers(Pike_interpreter.thread_id);
-  SWAP_OUT_THREAD(OBJ2THREAD(Pike_interpreter.thread_id));
-  OBJ2THREAD(Pike_interpreter.thread_id)->swapped=0;
-  OBJ2THREAD(Pike_interpreter.thread_id)->id=th_self();
+  state->id=th_self();
+  //  SWAP_OUT_THREAD(OBJ2THREAD(Pike_interpreter.thread_id));
   num_threads++;
   thread_table_insert(Pike_interpreter.thread_id);
+  state->status=THREAD_RUNNING;
 #endif
+  state->swapped = 0;
 #endif 
   SG(request_info).query_string = lookup_string_header("QUERY_STRING", 0);
   SG(server_context) = (void *)1; /* avoid server_context == NULL */
@@ -616,14 +669,16 @@ static void php_caudium_module_main(php_caudium_request *ureq)
   }
 
   /* Let PHP4 handle the deconding of the AUTH */
-  php_handle_auth_data(lookup_string_header("HTTP_AUTHORIZATION", NULL), SLS_C);
+  php_handle_auth_data(lookup_string_header("HTTP_AUTHORIZATION", NULL), TSRMLS_C);
    /* Swap out this thread and release the interpreter lock to allow
    * Pike threads to run. We wait since the above would otherwise require
    * a lot of unlock/lock.
    */
-#ifndef TEST_NO_THREADS
-  SWAP_OUT_CURRENT_THREAD();
+#ifndef USE_PIKE_LEVEL_THREADS
+  SWAP_OUT_THREAD(state);
   mt_unlock_interpreter();
+#else
+  THREADS_ALLOW();
 #endif
 
 #ifdef VIRTUAL_DIR
@@ -632,46 +687,48 @@ static void php_caudium_module_main(php_caudium_request *ureq)
    * isn't. Not a problem though, since it's on by default when using ZTS
    * which we require.
    */
-  V_CHDIR_FILE(THIS->filename->str);
+  VCWD_CHDIR_FILE(THIS->filename->str);
 #endif
   
   file_handle.type = ZEND_HANDLE_FILENAME;
   file_handle.filename = THIS->filename->str;
+  file_handle.opened_path = NULL;
   file_handle.free_filename = 0;
+
   THIS->written = 0;
-  res = php_request_startup(CLS_C ELS_CC PLS_CC SLS_CC);
+  res = php_request_startup(TSRMLS_C);
 
   if(res == FAILURE) {
     THREAD_SAFE_RUN({
-      
       apply_svalue(&THIS->done_cb, 0);
       pop_stack();
-      free_struct(SLS_C);
+      free_struct(TSRMLS_C);
     }, "Negative run response");
   } else {
-    THREAD_SAFE_RUN(php_caudium_hash_environment(CLS_C ELS_CC PLS_CC SLS_CC),
+    THREAD_SAFE_RUN(php_caudium_hash_environment(TSRMLS_C),
 		    "environment hashing");
-    php_execute_script(&file_handle CLS_CC ELS_CC PLS_CC);
+    php_execute_script(&file_handle TSRMLS_CC);
     php_request_shutdown(NULL);
     THREAD_SAFE_RUN({
       push_int(THIS->written);
       apply_svalue(&THIS->done_cb, 1);
       pop_stack();
-      free_struct(SLS_C);
+      free_struct(TSRMLS_C);
     }, "positive run response");
   }
-#ifndef TEST_NO_THREADS
+
+#ifndef USE_PIKE_LEVEL_THREADS
   mt_lock_interpreter();
-  SWAP_IN_CURRENT_THREAD();
+  SWAP_IN_THREAD(state);
 #if PIKE_MAJOR_VERSION == 7 && PIKE_MINOR_VERSION < 1
-  OBJ2THREAD(thread_id)->status=THREAD_EXITED;
-  co_signal(& OBJ2THREAD(thread_id)->status_change);
+  state->status=THREAD_EXITED;
+  co_signal(& state->status_change);
   thread_table_delete(thread_id);
   free_object(thread_id);
   thread_id=NULL;
 #else
-  OBJ2THREAD(Pike_interpreter.thread_id)->status=THREAD_EXITED;
-  co_signal(& OBJ2THREAD(Pike_interpreter.thread_id)->status_change);
+  state->status=THREAD_EXITED;
+  co_signal(& state->status_change);
   thread_table_delete(Pike_interpreter.thread_id);
   free_object(Pike_interpreter.thread_id);
   Pike_interpreter.thread_id=NULL;
@@ -679,6 +736,8 @@ static void php_caudium_module_main(php_caudium_request *ureq)
   cleanup_interpret();
   num_threads--;
   mt_unlock_interpreter();
+#else
+  THREADS_DISALLOW();
 #endif
 }
 
@@ -724,7 +783,7 @@ void f_php_caudium_request_handler(INT32 args)
       THIS->my_fd = fd;
   } else
     THIS->my_fd = 0;
-#ifdef TEST_NO_THREADS
+#ifdef USE_PIKE_LEVEL_THREADS
   php_caudium_module_main(THIS);
 #else
   th_farm((void (*)(void *))php_caudium_module_main, THIS);
@@ -732,7 +791,7 @@ void f_php_caudium_request_handler(INT32 args)
   pop_n_elems(args);
 }
 
-static void free_struct(SLS_D)
+static void free_struct(TSRMLS_D)
 {
   GET_THIS();
   if(THIS->request_data) free_mapping(THIS->request_data);
@@ -754,14 +813,14 @@ void pike_module_init( void )
   if (!caudium_php_initialized) {
     caudium_php_initialized = 1;
     tsrm_startup(1, 1, 0, NULL);
-    caudium_globals_id = ts_allocate_id(sizeof(php_caudium_request), NULL, NULL);
+    ts_allocate_id(&caudium_globals_id, sizeof(php_caudium_request), NULL, NULL);
     sapi_startup(&caudium_sapi_module);
     sapi_module.startup(&caudium_sapi_module);
     zend_startup_module(&php_caudium_module);
   }
   start_new_program(); /* Text */
   pike_add_function("run", f_php_caudium_request_handler,
-		    "function(string,mapping,object,function:void)", 0);
+		    "function(string, mapping, object, function:void)", 0);
   end_class("Interpreter", 0);
 }
 
