@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP version 4.0                                                      |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000 The PHP Group                   |
+   | Copyright (c) 1997-2001 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: roxen.c,v 1.36 2000/08/20 14:29:00 sas Exp $ */
+/* $Id: roxen.c,v 1.47.2.1 2001/10/12 11:56:37 ssb Exp $ */
 
 #include "php.h"
 #ifdef HAVE_ROXEN
@@ -211,15 +211,13 @@ INLINE static int lookup_integer_header(char *headername, int default_value)
  */
 
 static int
-php_roxen_low_ub_write(const char *str, uint str_length) {
+php_roxen_low_ub_write(const char *str, uint str_length TSRMLS_DC) {
   int sent_bytes = 0;
   struct pike_string *to_write = NULL;
-#ifdef ZTS
-  PLS_FETCH();
-#endif
 #ifdef ROXEN_USE_ZTS
   GET_THIS();
 #endif
+
   if(!MY_FD_OBJ->prog) {
     PG(connection_status) = PHP_CONNECTION_ABORTED;
     zend_bailout();
@@ -233,8 +231,7 @@ php_roxen_low_ub_write(const char *str, uint str_length) {
   pop_stack();
   if(sent_bytes != str_length) {
     /* This means the connection is closed. Dead. Gone. *sniff*  */
-    PG(connection_status) = PHP_CONNECTION_ABORTED;
-    zend_bailout();
+    php_handle_aborted_connection();
   }
   return sent_bytes;
 }
@@ -245,14 +242,12 @@ php_roxen_low_ub_write(const char *str, uint str_length) {
  */
 
 static int
-php_roxen_sapi_ub_write(const char *str, uint str_length)
+php_roxen_sapi_ub_write(const char *str, uint str_length TSRMLS_DC)
 {
-#ifdef ZTS
-  PLS_FETCH();
-#endif
 #ifdef ROXEN_USE_ZTS
   GET_THIS();
 #endif
+
   int sent_bytes = 0, fd = MY_FD;
   if(fd)
   {
@@ -279,7 +274,7 @@ php_roxen_sapi_ub_write(const char *str, uint str_length)
       }
     }
   } else {
-    THREAD_SAFE_RUN(sent_bytes = php_roxen_low_ub_write(str, str_length),
+    THREAD_SAFE_RUN(sent_bytes = php_roxen_low_ub_write(str, str_length TSRMLS_CC),
 		    "write");
   }
   return sent_bytes;
@@ -329,7 +324,7 @@ static void php_roxen_set_header(char *header_name, char *value, char *p)
  */
 static int
 php_roxen_sapi_header_handler(sapi_header_struct *sapi_header,
-			      sapi_headers_struct *sapi_headers SLS_DC)
+			      sapi_headers_struct *sapi_headers TSRMLS_DC)
 {
   char *header_name, *header_content, *p;
   header_name = sapi_header->header;
@@ -351,16 +346,14 @@ php_roxen_sapi_header_handler(sapi_header_struct *sapi_header,
  */
 
 static int
-php_roxen_low_send_headers(sapi_headers_struct *sapi_headers SLS_DC)
+php_roxen_low_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 {
-#ifdef ZTS
-  PLS_FETCH();
-#endif
   struct pike_string *ind;
   struct svalue *s_headermap;
 #ifdef ROXEN_USE_ZTS
   GET_THIS();
 #endif
+
   if(!MY_FD_OBJ->prog) {
     PG(connection_status) = PHP_CONNECTION_ABORTED;
     zend_bailout();
@@ -382,10 +375,10 @@ php_roxen_low_send_headers(sapi_headers_struct *sapi_headers SLS_DC)
 }
 
 static int
-php_roxen_sapi_send_headers(sapi_headers_struct *sapi_headers SLS_DC)
+php_roxen_sapi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 {
   int res = 0;
-  THREAD_SAFE_RUN(res = php_roxen_low_send_headers(sapi_headers SLS_CC), "send headers");
+  THREAD_SAFE_RUN(res = php_roxen_low_send_headers(sapi_headers TSRMLS_CC), "send headers");
   return res;
 }
 
@@ -400,9 +393,7 @@ INLINE static int php_roxen_low_read_post(char *buf, uint count_bytes)
 #ifdef ROXEN_USE_ZTS
   GET_THIS();
 #endif
-#ifdef ZTS
-  PLS_FETCH();
-#endif
+  TSRMLS_FETCH();
   
   if(!MY_FD_OBJ->prog)
   {
@@ -423,7 +414,7 @@ INLINE static int php_roxen_low_read_post(char *buf, uint count_bytes)
 }
 
 static int
-php_roxen_sapi_read_post(char *buf, uint count_bytes SLS_DC)
+php_roxen_sapi_read_post(char *buf, uint count_bytes TSRMLS_DC)
 {
   uint total_read = 0;
   THREAD_SAFE_RUN(total_read = php_roxen_low_read_post(buf, count_bytes), "read post");
@@ -436,7 +427,7 @@ php_roxen_sapi_read_post(char *buf, uint count_bytes SLS_DC)
  */
 	
 static char *
-php_roxen_sapi_read_cookies(SLS_D)
+php_roxen_sapi_read_cookies(TSRMLS_D)
 {
   char *cookies;
   cookies = lookup_string_header("HTTP_COOKIE", NULL);
@@ -447,7 +438,7 @@ static void php_info_roxen(ZEND_MODULE_INFO_FUNC_ARGS)
 {
   /*  char buf[512]; */
   php_info_print_table_start();
-  php_info_print_table_row(2, "SAPI module version", "$Id: roxen.c,v 1.36 2000/08/20 14:29:00 sas Exp $");
+  php_info_print_table_row(2, "SAPI module version", "$Id: roxen.c,v 1.47.2.1 2001/10/12 11:56:37 ssb Exp $");
   /*  php_info_print_table_row(2, "Build date", Ns_InfoBuildDate());
       php_info_print_table_row(2, "Config file path", Ns_InfoConfigFile());
       php_info_print_table_row(2, "Error Log path", Ns_InfoErrorLog());
@@ -468,6 +459,7 @@ static void php_info_roxen(ZEND_MODULE_INFO_FUNC_ARGS)
 }
 
 static zend_module_entry php_roxen_module = {
+  STANDARD_MODULE_HEADER,
   "Roxen",
   NULL,
   NULL,
@@ -475,6 +467,7 @@ static zend_module_entry php_roxen_module = {
   NULL,
   NULL,
   php_info_roxen,
+  NULL,
   STANDARD_MODULE_PROPERTIES
 };
 
@@ -490,7 +483,7 @@ static int php_roxen_startup(sapi_module_struct *sapi_module)
 
 /* this structure is static (as in "it does not change") */
 
-static sapi_module_struct sapi_module = {
+static sapi_module_struct roxen_sapi_module = {
   "roxen",
   "Roxen",
   php_module_startup,			/* startup */
@@ -529,7 +522,7 @@ static sapi_module_struct sapi_module = {
 			&pval, sizeof(zval *), NULL)
 
 static void
-php_roxen_hash_environment(CLS_D ELS_DC PLS_DC SLS_DC)
+php_roxen_hash_environment(TSRMLS_D)
 {
   int i;
   char buf[512];
@@ -579,18 +572,13 @@ php_roxen_hash_environment(CLS_D ELS_DC PLS_DC SLS_DC)
  * "executes" the script
  */
 
-static int php_roxen_module_main(SLS_D)
+static int php_roxen_module_main(TSRMLS_D)
 {
   int res, len;
   char *dir;
   zend_file_handle file_handle;
-#ifdef ZTS
-  CLS_FETCH();
-  PLS_FETCH();
-  ELS_FETCH();
 #ifdef ROXEN_USE_ZTS
   GET_THIS();
-#endif
 #endif
 
   file_handle.type = ZEND_HANDLE_FILENAME;
@@ -599,14 +587,14 @@ static int php_roxen_module_main(SLS_D)
   file_handle.opened_path = NULL;
 
   THREADS_ALLOW();
-  res = php_request_startup(CLS_C ELS_CC PLS_CC SLS_CC);
+  res = php_request_startup(TSRMLS_C);
   THREADS_DISALLOW();
   if(res == FAILURE) {
     return 0;
   }
-  php_roxen_hash_environment(CLS_C ELS_CC PLS_CC SLS_CC);
+  php_roxen_hash_environment(TSRMLS_C);
   THREADS_ALLOW();
-  php_execute_script(&file_handle CLS_CC ELS_CC PLS_CC);
+  php_execute_script(&file_handle TSRMLS_CC);
   php_request_shutdown(NULL);
   THREADS_DISALLOW();
   return 1;
@@ -624,10 +612,10 @@ void f_php_roxen_request_handler(INT32 args)
   struct svalue *done_callback, *raw_fd;
   struct pike_string *script, *ind;
   int status = 1;
-  SLS_FETCH();
 #ifdef ROXEN_USE_ZTS
   GET_THIS();
 #endif
+  TSRMLS_FETCH();
 
   if(current_thread == th_self())
     error("PHP4.Interpreter->run: Tried to run a PHP-script from a PHP "
@@ -660,11 +648,6 @@ void f_php_roxen_request_handler(INT32 args)
   SG(request_info).content_length = lookup_integer_header("HTTP_CONTENT_LENGTH", 0);
   SG(request_info).content_type = lookup_string_header("HTTP_CONTENT_TYPE", NULL);
   SG(sapi_headers).http_response_code = 200;
-  if (!strcmp(SG(request_info).request_method, "HEAD")) {
-    SG(request_info).headers_only = 1;
-  } else {
-    SG(request_info).headers_only = 0;
-  }
 
   /* FIXME: Check for auth stuff needs to be fixed... */ 
   SG(request_info).auth_user = NULL; 
@@ -681,7 +664,7 @@ void f_php_roxen_request_handler(INT32 args)
   } else
     THIS->my_fd = 0;
   
-  status = php_roxen_module_main(SLS_C);
+  status = php_roxen_module_main(TSRMLS_C);
   current_thread = -1;
   
   apply_svalue(done_callback, 0);
@@ -709,13 +692,13 @@ void pike_module_init( void )
 {
   if (!roxen_php_initialized) {
 #ifdef ZTS
-    tsrm_startup(1, 1, 0);
+    tsrm_startup(1, 1, 0, NULL);
 #ifdef ROXEN_USE_ZTS
-    roxen_globals_id = ts_allocate_id(sizeof(php_roxen_request), NULL, NULL);
+    ts_allocate_id(&roxen_globals_id, sizeof(php_roxen_request), NULL, NULL);
 #endif	 
 #endif
-    sapi_startup(&sapi_module);
-    php_roxen_startup(&sapi_module);
+    sapi_startup(&roxen_sapi_module);
+    php_roxen_startup(&roxen_sapi_module);
     roxen_php_initialized = 1;
     PHP_INIT_LOCK();
   }
@@ -723,7 +706,7 @@ void pike_module_init( void )
   ADD_STORAGE(php_roxen_request);
   set_init_callback(clear_struct);
   pike_add_function("run", f_php_roxen_request_handler,
-		    "function(string,mapping,object,function:int)", 0);
+		    "function(string, mapping, object, function:int)", 0);
   add_program_constant("Interpreter", (php_program = end_program()), 0);
 }
 
@@ -735,7 +718,7 @@ void pike_module_init( void )
 void pike_module_exit(void)
 {
   roxen_php_initialized = 0;
-  sapi_module.shutdown(&sapi_module);
+  roxen_sapi_module.shutdown(&roxen_sapi_module);
   if(php_program)  free_program(php_program);
 #ifdef ZTS
   tsrm_shutdown();
